@@ -20,6 +20,7 @@
 */
 
 #include "includes.h"
+#include "smbd/globals.h"
 #include "system/syslog.h"
 #include "system/filesys.h"
 #include "printing.h"
@@ -40,6 +41,7 @@
 #include "lib/util/string_wrappers.h"
 #include "lib/global_contexts.h"
 #include "source3/printing/rap_jobid.h"
+#include "source3/lib/substitute.h"
 
 extern userdom_struct current_user_info;
 
@@ -60,10 +62,6 @@ bool print_backend_init(struct messaging_context *msg_ctx)
 	bool ok;
 	char *print_cache_path;
 
-	if (!printer_list_parent_init()) {
-		return false;
-	}
-
 	print_cache_path = cache_path(talloc_tos(), "printing");
 	if (print_cache_path == NULL) {
 		return false;
@@ -1594,7 +1592,7 @@ static void print_queue_update(struct messaging_context *msg_ctx,
 			"",
 			get_current_gid(NULL),
 			get_current_username(),
-			current_user_info.domain,
+			get_current_user_info_domain(),
 			lpqcommand);
 	if (!lpqcommand) {
 		return;
@@ -1614,7 +1612,7 @@ static void print_queue_update(struct messaging_context *msg_ctx,
 			"",
 			get_current_gid(NULL),
 			get_current_username(),
-			current_user_info.domain,
+			get_current_user_info_domain(),
 			lprmcommand);
 	if (!lprmcommand) {
 		return;
@@ -2852,7 +2850,7 @@ NTSTATUS print_job_end(struct messaging_context *msg_ctx, int snum,
 				      "",
 				      get_current_gid(NULL),
 				      get_current_username(),
-				      current_user_info.domain,
+				      get_current_user_info_domain(),
 				      lpq_cmd);
 	if (lpq_cmd == NULL) {
 		status = NT_STATUS_PRINT_CANCELLED;
